#!/usr/bin/env python
#
# Copyright (C) 2014 - Renzo Davoli
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# 

import sys, json, Image, ImageDraw, argparse

inch=2.54
#inch=1

class mille:
	'''
	pcb object: size in mm, width/height in .1" elements.
	table: the same integer represents the same track
	drill: boolean (in the future values can represent different diameters)
	rear: for two sided pcb, not yet implemented
	'''
	def __init__(self):
		self.size=(0,0)
		self.width,self.height=0,0
		self.table=[]
		self.rear=[]
		self.drill=[]
	def valid(self,i,j):
		return True if i>0 and j>0 and i<self.width+1 and j<self.height+1 else False
	def new(self,size=(120,100),twosided=0):
		self.width,self.height=size
		self.size=size
		self.width,self.height=int(self.width//inch),int(self.height//inch)
		self.drill=[[0 for _ in range(self.width+2)] for _ in range(self.height+2)]
		self.table=[[1 if self.valid(i,j) else 0 for i in range(self.width+2)]
			for j in range(self.height+2)]
		if twosided:
			self.rear=[[1 if self.valid(i,j) else 0 for i in range(self.width+2)]
				for j in range(self.height+2)]
	def reversei(self):
		for i in range((self.height+2)//2):
			self.table[i],self.table[self.height+1-i]=self.table[self.height+1-i],self.table[i]
			self.drill[i],self.drill[self.height+1-i]=self.drill[self.height+1-i],self.drill[i]
			if self.rear:
				self.rear[i],self.rear[self.height+1-i]=self.rear[self.height+1-i],self.rear[i]
	def reversej(self):
		for i in range(self.height+2):
			for j in range((self.width+2)//2):
				self.table[i][j],self.table[i][self.width+1-j]=self.table[i][self.width+1-j],self.table[i][j]
				self.drill[i][j],self.drill[i][self.width+1-j]=self.drill[i][self.width+1-j],self.drill[i][j]
				if self.rear:
					self.rear[i][j],self.rear[i][self.width+1-j]=self.rear[i][self.width+1-j],self.rear[i][j]


WHITE="rgb(255,255,255)"
GRAY="rgb(230,230,230)"
GREEN="rgb(100,230,100)"
RED="rgb(230,100,100)"
HIGHLIGHT="rgb(100,230,230)"
BLACK="rgb(0,0,0)"

def main():
	pcb=mille()
	parser = argparse.ArgumentParser()
	parser.add_argument("input_file", help="millefori input file")
	parser.add_argument("output_file", help="output file (.png, .pdf ...)")
	parser.add_argument("-r", "--reverse", action="store_true",
                    help="soldering side (CNC working side for front)")
	parser.add_argument("-2", "--backside", action="store_true",
			              help="backside (component side)")
	args = parser.parse_args()
	pcb.filename=args.input_file
	imgfile=args.output_file
	if file_load(pcb) == False:
		print "Error opening file", filename
	if args.reverse:
		pcb.reversej()
	if args.backside:
		img=computeimg(pcb,pcb.rear)
	else:
		img=computeimg(pcb,pcb.table)
	img.save(imgfile)

def computeimg(pcb,table):
	img=Image.new("RGB", (pcb.width*10+20,pcb.height*10+20), "white")
	draw = ImageDraw.Draw(img)
	for i in range(pcb.width+1):
		for j in range(pcb.height+1):
			if (table[j][i+1] == table[j+1][i+1]):
				color=GRAY
			else:
				color=BLACK
			if (i<pcb.width):
				draw.line([(i*10+10,j*10+10),(i*10+20,j*10+10)],fill=color)
			if (table[j+1][i] == table[j+1][i+1]):
				color=GRAY
			else:
			 color=BLACK
			if (j<pcb.height):
				draw.line([(i*10+10,j*10+10),(i*10+10,j*10+20)],fill=color)
			draw.point([(i*10+10,j*10+10)],fill=BLACK)
			if pcb.drill[j][i]:
				draw.line([(i*10+4,j*10+4),(i*10+6,j*10+6)],fill=BLACK)
				draw.line([(i*10+6,j*10+4),(i*10+4,j*10+6)],fill=BLACK)
	del draw
	return img

def file_load(pcb):
  try:
    try:
      name=pcb.filename
      with open(name, "r") as f:
				loaded=json.load(f)
				for f in loaded:
					pcb.__dict__[f]=loaded[f]
      pcb.filename=name
    except TypeError:
      #old format
      with open(pcb.filename, "r") as f:
        pcb.size,pcb.table,pcb.drill=json.load(f)
      pcb.height=len(pcb.table)-2
      pcb.width=len(pcb.table[0])-2
  except IOError:
    return False
  return True

if __name__ == '__main__':
  main()

